@@ -23,11 +23,13 @@ import java.awt.geom.AffineTransform;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.io.IOException;
 import java.io.ObjectInputStream;
+import java.lang.ref.WeakReference;
 import java.math.BigDecimal;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 
@@ -51,8 +53,11 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   private float                      fixedHeight;
   private float                      dropOnTopElevation; 
   private String                     currency;
-  private List<Integer>              furnitureDefaultColors;
-  private List<HomeTexture>          furnitureDefaultTextures;
+  // Fields removed in version 5.2
+  // private List<Integer>           furnitureDefaultColors;
+  // private List<HomeTexture>       furnitureDefaultTextures;
+
+  private transient PropertyChangeListener furnitureListener;
 
   /**
    * Creates a group from the given <code>furniture</code> list. 
@@ -74,11 +79,123 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public HomeFurnitureGroup(List<HomePieceOfFurniture> furniture,
                             HomePieceOfFurniture leadingPiece,
                             String name) {
+    this(furniture, leadingPiece.getAngle(), false, name);
+  }
+  
+  /**
+   * Creates a group from the given <code>furniture</code> list. 
+   * The level of each piece of furniture of the group will be reset to <code>null</code> and if they belong to levels
+   * with different elevations, their elevation will be updated to be relative to the elevation of the lowest level.
+   * @since 5.2
+   */
+  public HomeFurnitureGroup(List<HomePieceOfFurniture> furniture,
+                            float angle, boolean modelMirrored,
+                            String name) {
     super(furniture.get(0));
     this.furniture = Collections.unmodifiableList(furniture); 
     
+    boolean movable = true;
+    boolean visible = false;
+    for (HomePieceOfFurniture piece : furniture) {
+      movable &= piece.isMovable();
+      visible |= piece.isVisible();
+    }
+    
+    setName(name);
+    setNameVisible(false);
+    setNameXOffset(0);
+    setNameYOffset(0);
+    setNameStyle(null);
+    setDescription(null);
+    super.setMovable(movable);
+    setVisible(visible);
+   
+    updateLocationAndSize(furniture, angle, true);
+    super.setAngle(angle);
+    super.setModelMirrored(modelMirrored);
+
+    addFurnitureListener();
+  }
+
+  /**
+   * Initializes new piece fields to their default values 
+   * and reads piece from <code>in</code> stream with default reading method.
+   */
+  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
+    this.dropOnTopElevation = -1;
+    this.deformable = true;
+    this.texturable = true;
+    in.defaultReadObject();
+    addFurnitureListener();
+  }
+
+  /**
+   * Updates the location and size of this group from the furniture it contains.
+   */
+  private void updateLocationAndSize(List<HomePieceOfFurniture> furniture,
+                                     float angle,
+                                     boolean init) {
+    // Update abilities of the group according to its furniture
+    this.resizable = true;
+    this.deformable = true;
+    this.texturable = true;
+    this.doorOrWindow = true;
+    this.currency = furniture.get(0).getCurrency();
+    for (HomePieceOfFurniture piece : furniture) {
+      this.resizable &= piece.isResizable();
+      this.deformable &= piece.isDeformable();
+      this.texturable &= piece.isTexturable();
+      this.doorOrWindow &= piece.isDoorOrWindow();
+      if (this.currency != null) {
+        if (piece.getCurrency() == null
+            || !piece.getCurrency().equals(this.currency)) {
+          this.currency = null; 
+        }
+      }
+    }
+
+    float elevation = Float.MAX_VALUE;
+    if (init) {
+      // Search the lowest level elevation among grouped furniture
+      Level minLevel = null;
+      for (HomePieceOfFurniture piece : furniture) {
+        Level level = piece.getLevel();
+        if (level != null 
+            && (minLevel == null
+                || level.getElevation() < minLevel.getElevation())) {
+          minLevel = level;
+        }
+      }
+      for (HomePieceOfFurniture piece : furniture) {
+        if (piece.getLevel() != null) {
+          elevation = Math.min(elevation, piece.getGroundElevation() - minLevel.getElevation());
+          // Reset piece level and elevation
+          piece.setElevation(piece.getGroundElevation() - minLevel.getElevation());
+          piece.setLevel(null);
+        } else {
+          elevation = Math.min(elevation, piece.getElevation());
+        }
+      }
+    } else {
+      for (HomePieceOfFurniture piece : furniture) {
+        elevation = Math.min(elevation, piece.getElevation());
+      }
+    }
+
+    float height = 0;
+    float dropOnTopElevation = -1;
+    for (HomePieceOfFurniture piece : furniture) {
+      height = Math.max(height, piece.getElevation() + piece.getHeightInPlan());
+      if (piece.getDropOnTopElevation() >= 0) {
+        dropOnTopElevation = Math.max(dropOnTopElevation, 
+            piece.getElevation() + piece.getHeightInPlan() * piece.getDropOnTopElevation());
+      }
+    }
+    height -= elevation;
+    dropOnTopElevation -= elevation;
+
     // Search the size of the furniture group
-    AffineTransform rotation = AffineTransform.getRotateInstance(-leadingPiece.getAngle());
+    AffineTransform rotation = AffineTransform.getRotateInstance(-angle);
     Rectangle2D unrotatedBoundingRectangle = null;
     for (HomePieceOfFurniture piece : getFurnitureWithoutGroups(furniture)) {
       GeneralPath pieceShape = new GeneralPath();
@@ -96,101 +213,67 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     }
     // Search center of the group
     Point2D center = new Point2D.Float((float)unrotatedBoundingRectangle.getCenterX(), (float)unrotatedBoundingRectangle.getCenterY());
-    rotation.setToRotation(leadingPiece.getAngle());
+    rotation.setToRotation(angle);
     rotation.transform(center, center);
-    
-    float elevation = Float.MAX_VALUE;
-    float height    = 0;
-    boolean movable = true;
-    this.resizable = true;
-    this.deformable = true;
-    this.texturable = true;
-    this.doorOrWindow = true;
-    boolean visible = false;
-    boolean modelMirrored = true;
-    this.dropOnTopElevation = -1;
-    this.currency = furniture.get(0).getCurrency();
-    // Search the lowest level elevation among grouped furniture
-    Level minLevel = null;
-    for (HomePieceOfFurniture piece : furniture) {
-      Level level = piece.getLevel();
-      if (level != null 
-          && (minLevel == null
-              || level.getElevation() < minLevel.getElevation())) {
-        minLevel = level;
-      }
-    }
-    for (HomePieceOfFurniture piece : furniture) {
-      Level level = piece.getLevel();
-      if (level != null) {
-        elevation = Math.min(elevation, piece.getGroundElevation() - minLevel.getElevation());
-      } else {
-        elevation = Math.min(elevation, piece.getElevation());
-      }
-    }
-    for (HomePieceOfFurniture piece : furniture) {
-      if (piece.getLevel() != null) {
-        piece.setElevation(piece.getGroundElevation() - minLevel.getElevation());
-      }
-      piece.setLevel(null);
-      height = Math.max(height, piece.getElevation() + piece.getHeight());
-      if (piece.getDropOnTopElevation() >= 0) {
-        this.dropOnTopElevation = Math.max(this.dropOnTopElevation, 
-            piece.getElevation() + piece.getHeight() * piece.getDropOnTopElevation());
-      }
-      movable &= piece.isMovable();
-      this.resizable &= piece.isResizable();
-      this.deformable &= piece.isDeformable();
-      this.texturable &= piece.isTexturable();
-      this.doorOrWindow &= piece.isDoorOrWindow();
-      visible |= piece.isVisible();
-      modelMirrored &= piece.isModelMirrored();
 
-      if (this.currency != null) {
-        if (piece.getCurrency() == null
-            || !piece.getCurrency().equals(this.currency)) {
-          this.currency = null; 
-        }
-      }
-    }
     if (this.resizable) {
-      super.setWidth((float)unrotatedBoundingRectangle.getWidth());
-      super.setDepth((float)unrotatedBoundingRectangle.getHeight());
-      super.setHeight(height - elevation);
-      super.setModelMirrored(modelMirrored);
+      float width = (float)unrotatedBoundingRectangle.getWidth();
+      super.setWidth(width);
+      super.setWidthInPlan(width);
+      float depth = (float)unrotatedBoundingRectangle.getHeight();
+      super.setDepth(depth);
+      super.setDepthInPlan(depth);
+      super.setHeight(height);
+      super.setHeightInPlan(height);
     } else {
       this.fixedWidth = (float)unrotatedBoundingRectangle.getWidth();
       this.fixedDepth = (float)unrotatedBoundingRectangle.getHeight();
-      this.fixedHeight = height - elevation;
-    }
-    this.dropOnTopElevation = (this.dropOnTopElevation - elevation) / height;
-    setName(name);
-    setNameVisible(false);
-    setNameXOffset(0);
-    setNameYOffset(0);
-    setNameStyle(null);
-    setDescription(null);
-    setMovable(movable);
-    setVisible(visible);
-    if (this.texturable) {
-      super.setColor(null);
-      super.setTexture(null);
+      this.fixedHeight = height;
     }
+    this.dropOnTopElevation = dropOnTopElevation / height;
     super.setX((float)center.getX());
     super.setY((float)center.getY());
-    super.setAngle(leadingPiece.getAngle());
     super.setElevation(elevation);
   }
+  
+  /**
+   * Adds a listener to the furniture of this group that will update the size and location 
+   * of the group when its furniture is moved or resized. 
+   */
+  private void addFurnitureListener() {
+    this.furnitureListener = new LocationAndSizeChangeListener(this);
+    for (HomePieceOfFurniture piece : this.furniture) {
+      piece.addPropertyChangeListener(this.furnitureListener);
+    }    
+  }
 
   /**
-   * Initializes new piece fields to their default values 
-   * and reads piece from <code>in</code> stream with default reading method.
+   * Properties listener that updates the size and location of this group. 
+   * This listener is bound to this group with a weak reference to avoid a strong link 
+   * of this group towards the furniture it contains.  
    */
-  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
-    this.dropOnTopElevation = -1;
-    this.deformable = true;
-    this.texturable = true;
-    in.defaultReadObject();
+  private static class LocationAndSizeChangeListener implements PropertyChangeListener { 
+    private WeakReference<HomeFurnitureGroup> group;
+    
+    public LocationAndSizeChangeListener(HomeFurnitureGroup group) {
+      this.group = new WeakReference<HomeFurnitureGroup>(group);
+    }
+    
+    public void propertyChange(PropertyChangeEvent ev) {
+      // If the group was garbage collected, remove this listener from furniture
+      final HomeFurnitureGroup group = this.group.get();
+      if (group == null) {
+        ((HomePieceOfFurniture)ev.getSource()).removePropertyChangeListener(this);
+      } else if (HomePieceOfFurniture.Property.X.name().equals(ev.getPropertyName())
+          || HomePieceOfFurniture.Property.Y.name().equals(ev.getPropertyName())
+          || HomePieceOfFurniture.Property.ELEVATION.name().equals(ev.getPropertyName())
+          || HomePieceOfFurniture.Property.ANGLE.name().equals(ev.getPropertyName())
+          || HomePieceOfFurniture.Property.WIDTH_IN_PLAN.name().equals(ev.getPropertyName())
+          || HomePieceOfFurniture.Property.DEPTH_IN_PLAN.name().equals(ev.getPropertyName())
+          || HomePieceOfFurniture.Property.HEIGHT_IN_PLAN.name().equals(ev.getPropertyName())) {
+        group.updateLocationAndSize(group.getFurniture(), group.getAngle(), false);
+      }
+    }
   }
 
   /**
@@ -208,13 +291,59 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     return pieces;
   }
   
+  /**
+   * Returns the furniture of this group and of all its subgroups.  
+   * @since 5.0
+   */
+  public List<HomePieceOfFurniture> getAllFurniture() {
+    List<HomePieceOfFurniture> pieces = new ArrayList<HomePieceOfFurniture>(this.furniture);
+    for (HomePieceOfFurniture piece : getFurniture()) {
+      if (piece instanceof HomeFurnitureGroup) {
+        pieces.addAll(((HomeFurnitureGroup)piece).getAllFurniture());
+      } 
+    }
+    return pieces;
+  }
+  
   /**
    * Returns an unmodifiable list of the furniture of this group.
    */
   public List<HomePieceOfFurniture> getFurniture() {
-    return this.furniture;
+    return Collections.unmodifiableList(this.furniture);
   }
   
+  /**
+   * Adds the <code>piece</code> in parameter at the given <code>index</code>.
+   */
+  void addPieceOfFurniture(HomePieceOfFurniture piece, int index) {
+    // Make a copy of the list to avoid conflicts in the list returned by getFurniture
+    this.furniture = new ArrayList<HomePieceOfFurniture>(this.furniture);
+    piece.setLevel(getLevel());
+    this.furniture.add(index, piece);
+    piece.addPropertyChangeListener(this.furnitureListener);
+    updateLocationAndSize(this.furniture, getAngle(), false);
+  }
+
+  /**
+   * Deletes the <code>piece</code> in parameter from this group.
+   * @throws IllegalStateException if the last piece in this group is the one in parameter
+   */
+  void deletePieceOfFurniture(HomePieceOfFurniture piece) {
+    int index = this.furniture.indexOf(piece);
+    if (index != -1) {
+      if (this.furniture.size() > 1) {
+        piece.setLevel(null);
+        piece.removePropertyChangeListener(this.furnitureListener);
+        // Make a copy of the list to avoid conflicts in the list returned by getFurniture
+        this.furniture = new ArrayList<HomePieceOfFurniture>(this.furniture);
+        this.furniture.remove(index);
+        updateLocationAndSize(this.furniture, getAngle(), false);
+      } else {
+        throw new IllegalStateException("Group can't be empty");
+      }
+    }
+  }
+
   /**
    * Returns <code>null</code>.
    */
@@ -233,7 +362,7 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   }
   
   /**
-   * Returns <code>true</code> if all furniture of this group are movable.
+   * Returns <code>true</code> if this group is movable.
    */
   @Override
   public boolean isMovable() {
@@ -241,15 +370,12 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   }
   
   /**
-   * Sets whether this piece is movable or not.
+   * Sets whether this group is movable or not.
    * @since 3.1
    */
   @Override
   public void setMovable(boolean movable) {
     super.setMovable(movable);
-    for (HomePieceOfFurniture piece : this.furniture) {
-      piece.setMovable(movable);
-    }
   }
 
   /**
@@ -286,6 +412,15 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     return this.texturable;
   }
   
+  /**
+   * Returns <code>false</code>.
+   * @since 5.5
+   */
+  @Override
+  public boolean isHorizontallyRotatable() {
+    return false;
+  }
+  
   /**
    * Returns the width of this group.
    */
@@ -298,6 +433,16 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     }
   }
   
+  /**
+   * Returns the width of this group. As a group can't be rotated around an horizontal axis, 
+   * its width in the horizontal plan is equal to its width.
+   * @since 5.5
+   */
+  @Override
+  public float getWidthInPlan() {
+    return getWidth();
+  }
+
   /**
    * Returns the depth of this group.
    */
@@ -310,6 +455,16 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     }
   }
   
+  /**
+   * Returns the depth of this group. As a group can't be rotated around an horizontal axis, 
+   * its depth in the horizontal plan is equal to its depth.
+   * @since 5.5
+   */
+  @Override
+  public float getDepthInPlan() {
+    return getDepth();
+  }
+  
   /**
    * Returns the height of this group.
    */
@@ -321,7 +476,35 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
       return super.getHeight();
     }
   }
-  
+
+  /**
+   * Returns the height of this group. As a group can't be rotated around an horizontal axis, 
+   * its height in the horizontal plan is equal to its height.
+   * @since 5.5
+   */
+  @Override
+  public float getHeightInPlan() {
+    return getHeight();
+  }
+
+  /**
+   * Returns <code>true</code> if this piece or a child of this group is rotated around an horizontal axis.
+   * @since 5.5
+   */
+  @Override
+  public boolean isHorizontallyRotated() {
+    if (super.isHorizontallyRotated()) {
+      return true;
+    } else {
+      for (HomePieceOfFurniture childPiece : getFurniture()) {
+        if (childPiece.isHorizontallyRotated()) {
+          return true;
+        }
+      }
+      return false;
+    }
+  }
+
   /**
    * Returns the elevation at which should be placed an object dropped on this group.
    * @since 4.4 
@@ -341,6 +524,7 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   /**
    * Returns <code>null</code>.
    */
+  @Override
   public Content getPlanIcon() {
     return null;
   }
@@ -348,10 +532,28 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   /**
    * Returns <code>null</code>.
    */
+  @Override
   public Content getModel() {
     return null;
   }
   
+  /**
+   * @throws IllegalStateException
+   */
+  @Override
+  public void setModelSize(Long modelSize) {
+    throw new IllegalStateException("Can't set model size of a group");
+  }
+  
+  /**
+   * Returns <code>null</code>.
+   * @since 5.5
+   */
+  @Override
+  public Long getModelSize() {
+    return null;
+  }
+  
   /**
    * Returns an identity matrix.
    */
@@ -359,7 +561,34 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public float [][] getModelRotation() {
     return IDENTITY;
   }
+  
+  /**
+   * Returns <code>true</code>.
+   * @since 5.5
+   */
+  @Override
+  public boolean isModelCenteredAtOrigin() {
+    return true;
+  }
 
+  /**
+   * Returns 0.
+   * @since 5.5
+   */
+  @Override
+  public float getPitch() {
+    return 0;
+  }
+  
+  /**
+   * Returns 0.
+   * @since 5.5
+   */
+  @Override
+  public float getRoll() {
+    return 0;
+  }
+  
   /**
    * Returns <code>null</code>.
    * @since 3.5
@@ -491,69 +720,87 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     return false;
   }
   
+  /**
+   * Returns <code>null</code>.
+   * @since 5.2
+   */
+  @Override
+  public Integer getColor() {
+    return null;
+  }
+  
   /**
    * Sets the <code>color</code> of the furniture of this group.
    */
   @Override
   public void setColor(Integer color) {
-    super.setColor(color);
-    if (color != null) {      
-      storeDefaultColorsAndTextures();
+    if (isTexturable()) {
       for (HomePieceOfFurniture piece : this.furniture) {
-        piece.setTexture(null);
         piece.setColor(color);
-      } 
-    } else if (getTexture() == null) {
-      restoreDefaultColorsAndTextures();
-    }
+      }
+    } 
   }
 
+  /**
+   * Returns <code>null</code>.
+   * @since 5.2
+   */
+  @Override
+  public HomeTexture getTexture() {
+    return null;
+  }
+  
   /**
    * Sets the <code>texture</code> of the furniture of this group.
    */
   @Override
   public void setTexture(HomeTexture texture) {
-    super.setTexture(texture);
-    if (texture != null) {      
-      storeDefaultColorsAndTextures();
+    if (isTexturable()) {
       for (HomePieceOfFurniture piece : this.furniture) {
-        piece.setColor(null);
         piece.setTexture(texture);
-      } 
-    } else if (getColor() == null) {
-      restoreDefaultColorsAndTextures();
-    }
-  }
-  
-  /**
-   * Stores default colors and textures. 
-   */
-  private void storeDefaultColorsAndTextures() {
-    if (this.furnitureDefaultColors == null) {
-      // Retrieve default color and texture of child furniture
-      Integer [] furnitureDefaultColors = new Integer [this.furniture.size()];
-      HomeTexture [] furnitureDefaultTextures = new HomeTexture [this.furniture.size()];
-      for (int i = 0; i < this.furniture.size(); i++) {
-        furnitureDefaultColors [i] = this.furniture.get(i).getColor();
-        furnitureDefaultTextures [i] = this.furniture.get(i).getTexture();
-      } 
-      this.furnitureDefaultColors = Arrays.asList(furnitureDefaultColors);
-      this.furnitureDefaultTextures = Arrays.asList(furnitureDefaultTextures);
-    }
-  }
-  
-  /**
-   * Restores default colors and textures
-   */
-  private void restoreDefaultColorsAndTextures() {
-    if (this.furnitureDefaultColors != null) {
-      for (int i = 0; i < this.furniture.size(); i++) {
-        this.furniture.get(i).setColor(this.furnitureDefaultColors.get(i));
-        this.furniture.get(i).setTexture(this.furnitureDefaultTextures.get(i));
       }
-      this.furnitureDefaultColors = null;
-      this.furnitureDefaultTextures = null;
-    }
+    } 
+  }
+  
+  /**
+   * Returns <code>null</code>.
+   * @since 5.2
+   */
+  @Override
+  public HomeMaterial [] getModelMaterials() {
+    return null;
+  }
+
+  /**
+   * Sets the materials of the furniture of this group.
+   */
+  @Override
+  public void setModelMaterials(HomeMaterial [] modelMaterials) {
+    if (isTexturable()) {
+      for (HomePieceOfFurniture piece : this.furniture) {
+        piece.setModelMaterials(modelMaterials);
+      }
+    } 
+  }
+  
+  /**
+   * Returns <code>null</code>.
+   * @since 5.2
+   */
+  public Float getShininess() {
+    return null;
+  }
+  
+  /**
+   * Sets the shininess of the furniture of this group.
+   * @since 5.2
+   */
+  public void setShininess(Float shininess) {
+    if (isTexturable()) {
+      for (HomePieceOfFurniture piece : this.furniture) {
+        piece.setShininess(shininess);
+      }
+    } 
   }
 
   /**
@@ -563,16 +810,18 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public void setAngle(float angle) {
     if (angle != getAngle()) {
       float angleDelta = angle - getAngle();
-      super.setAngle(angle);
       double cosAngleDelta = Math.cos(angleDelta);
       double sinAngleDelta = Math.sin(angleDelta);
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         piece.setAngle(piece.getAngle() + angleDelta);     
         float newX = getX() + (float)((piece.getX() - getX()) * cosAngleDelta - (piece.getY() - getY()) * sinAngleDelta);
         float newY = getY() + (float)((piece.getX() - getX()) * sinAngleDelta + (piece.getY() - getY()) * cosAngleDelta);
         piece.setX(newX);
         piece.setY(newY);
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setAngle(angle);
     }
   }
   
@@ -583,10 +832,12 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public void setX(float x) {
     if (x != getX()) {
       float dx = x - getX();
-      super.setX(x);
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         piece.setX(piece.getX() + dx);
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setX(x);
     }
   }
 
@@ -597,23 +848,26 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public void setY(float y) {
     if (y != getY()) {
       float dy = y - getY();
-      super.setY(y);
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.addPropertyChangeListener(this.furnitureListener);
         piece.setY(piece.getY() + dy);
+        piece.removePropertyChangeListener(this.furnitureListener);
       }
+      super.setY(y);
     }
   }
 
   /**
    * Sets the <code>width</code> of this group, then moves and resizes its furniture accordingly.
+   * This method shouldn't be called on a group that contain furniture rotated around an horizontal axis.
    */
   @Override
   public void setWidth(float width) {
     if (width != getWidth()) {
       float widthFactor = width / getWidth();
-      super.setWidth(width);
       float angle = getAngle();
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         float angleDelta = piece.getAngle() - angle;
         float pieceWidth = piece.getWidth();
         float pieceDepth = piece.getDepth();
@@ -629,20 +883,23 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
         // Rotate piece back to its angle
         piece.setX(getX() + (float)((newX - getX()) * cosAngle - (newY - getY()) * sinAngle));
         piece.setY(getY() + (float)((newX - getX()) * sinAngle + (newY - getY()) * cosAngle));
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setWidth(width);
     }
   }
-
+  
   /**
    * Sets the <code>depth</code> of this group, then moves and resizes its furniture accordingly.
+   * This method shouldn't be called on a group that contain furniture rotated around an horizontal axis.
    */
   @Override
   public void setDepth(float depth) {
     if (depth != getDepth()) {
       float depthFactor = depth / getDepth();
-      super.setDepth(depth);
       float angle = getAngle();
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         float angleDelta = piece.getAngle() - angle;
         float pieceWidth = piece.getWidth();
         float pieceDepth = piece.getDepth();
@@ -658,26 +915,62 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
         // Rotate piece back to its angle
         piece.setX(getX() + (float)((newX - getX()) * cosAngle - (newY - getY()) * sinAngle));
         piece.setY(getY() + (float)((newX - getX()) * sinAngle + (newY - getY()) * cosAngle));
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setDepth(depth);
     }
   }
 
   /**
    * Sets the <code>height</code> of this group, then moves and resizes its furniture accordingly.
+   * This method shouldn't be called on a group that contain furniture rotated around an horizontal axis.
    */
   @Override
   public void setHeight(float height) {
     if (height != getHeight()) {
       float heightFactor = height / getHeight();
-      super.setHeight(height);
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         piece.setHeight(piece.getHeight() * heightFactor);
         piece.setElevation(getElevation() 
             + (piece.getElevation() - getElevation()) * heightFactor);
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setHeight(height);
     }
   }
 
+  /**
+   * Scales this group and its children with the given <code>ratio</code>.
+   * @since 5.5
+   */
+  @Override
+  public void scale(float scale) {
+    float angle = getAngle();
+    for (HomePieceOfFurniture piece : this.furniture) {
+      piece.removePropertyChangeListener(this.furnitureListener);
+      piece.setWidth(piece.getWidth() * scale);
+      piece.setDepth(piece.getDepth() * scale);
+      piece.setHeight(piece.getHeight() * scale);
+      // Rotate piece to angle 0
+      double cosAngle = Math.cos(angle);
+      double sinAngle = Math.sin(angle);
+      float newX = getX() + (float)((piece.getX() - getX()) * cosAngle + (piece.getY() - getY()) * sinAngle);
+      float newY = getY() + (float)((piece.getX() - getX()) * -sinAngle + (piece.getY() - getY()) * cosAngle);
+      // Update its coordinates
+      newX = getX() + (newX - getX()) * scale; 
+      newY = getY() + (newY - getY()) * scale;
+      // Rotate piece back to its angle
+      piece.setX(getX() + (float)((newX - getX()) * cosAngle - (newY - getY()) * sinAngle));
+      piece.setY(getY() + (float)((newX - getX()) * sinAngle + (newY - getY()) * cosAngle));
+      piece.setElevation(getElevation() + (piece.getElevation() - getElevation()) * scale);
+      piece.addPropertyChangeListener(this.furnitureListener);
+    }
+    super.setWidth(getWidth() * scale);
+    super.setDepth(getDepth() * scale);
+    super.setHeight(getHeight() * scale);
+  }
+
   /**
    * Sets the <code>elevation</code> of this group, then moves its furniture accordingly.
    */
@@ -685,10 +978,12 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public void setElevation(float elevation) {
     if (elevation != getElevation()) {
       float elevationDelta = elevation - getElevation();
-      super.setElevation(elevation);
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         piece.setElevation(piece.getElevation() + elevationDelta);
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setElevation(elevation);
     }
   }
   
@@ -698,9 +993,9 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   @Override
   public void setModelMirrored(boolean modelMirrored) {
     if (modelMirrored != isModelMirrored()) {
-      super.setModelMirrored(modelMirrored);
       float angle = getAngle();
       for (HomePieceOfFurniture piece : this.furniture) {
+        piece.removePropertyChangeListener(this.furnitureListener);
         piece.setModelMirrored(!piece.isModelMirrored());
         // Rotate piece to angle 0
         double cosAngle = Math.cos(angle);
@@ -712,7 +1007,9 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
         // Rotate piece back to its angle
         piece.setX(getX() + (float)((newX - getX()) * cosAngle - (newY - getY()) * sinAngle));
         piece.setY(getY() + (float)((newX - getX()) * sinAngle + (newY - getY()) * cosAngle));
+        piece.addPropertyChangeListener(this.furnitureListener);
       }
+      super.setModelMirrored(modelMirrored);
     }
   }
   
@@ -721,10 +1018,10 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
    */
   @Override
   public void setVisible(boolean visible) {
-    super.setVisible(visible);
     for (HomePieceOfFurniture piece : this.furniture) {
       piece.setVisible(visible);
     }
+    super.setVisible(visible);
   }
 
   /**
@@ -732,10 +1029,10 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
    */
   @Override
   public void setLevel(Level level) {
-    super.setLevel(level);
     for (HomePieceOfFurniture piece : this.furniture) {
       piece.setLevel(level);
     }
+    super.setLevel(level);
   }
   
   /**
@@ -788,12 +1085,7 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
       }
     }
     clone.furniture = Collections.unmodifiableList(clone.furniture);
-    if (this.furnitureDefaultColors != null)  {
-      clone.furnitureDefaultColors = new ArrayList<Integer>(this.furnitureDefaultColors);
-    }
-    if (this.furnitureDefaultTextures != null) {
-      clone.furnitureDefaultTextures = new ArrayList<HomeTexture>(this.furnitureDefaultTextures);
-    }
+    clone.addFurnitureListener();
     return clone;
   }
 }
